#This R script visualizes an elbow plot.
#-----------------------------------------
library("tm")
library("cluster")

#-------------------------------
#Create corpus from *.txt files 
#-------------------------------
cname <- file.path("#your directory containing the .txt file#")

corpus <- Corpus(DirSource(cname))                                                #create corpus                                            
corpus <- tm_map(corpus, content_transformer(tolower))                            #convert characters to lower case
corpus <- tm_map(corpus, removePunctuation, preserve_intra_word_dashes = TRUE)    #remove Punctuation without intra word dashes
corpus <- tm_map(corpus, removeWords, stopwords("english"))                       #remove stopwords
corpus <- tm_map(corpus, removeNumbers)                                           #remove numbers
corpus <- tm_map(corpus, content_transformer(stripWhitespace))                    #remove whitespace; do this at the end of preprocessing (again)!

dt.mat <- DocumentTermMatrix(corpus)                                              #create DocumentTermMatrix
d <- as.data.frame(as.matrix(dt.mat))

setwd("#your directory for saving the elbow plot#")

runs <- 10
wss_plot <- c("1")
wss <- (nrow(d)-1)*sum(apply(d,2,var))
for (i in 1:20) {
  for (j in 1:runs){
    wss[j] <- sum(kmeans(d, centers=i)$withinss)
  }
  wss_plot[i] <- sum(wss)/runs
}

pdf("elbow_plot.pdf", width = 12)
plot(1:20, wss_plot, type="b", xlab="Number of Clusters", ylab="within groups sum of squares")
dev.off()

