#This R script implements an algorithm for determining keywordlists consisting of frequent terms (onegrams and bigrams)
#and extracts sentences based on these lists.
#-----------------------------------------------------------------------------------------------------------------------

#load libraries
library("tm")
library("stringr")
library("slam")
library("NLP")
library("openNLP")

#initialize corpus and preprocess
cname <- file.path("#your directory containing the *.txt file#")

corpus <- Corpus(DirSource(cname))                                                #create corpus                                            
corpus <- tm_map(corpus, content_transformer(tolower))                            #convert characters to lower case
corpus <- tm_map(corpus, removeWords, c("european journal of cancer"))            #remove customized words (depending on the document!)
corpus <- tm_map(corpus, removeWords, stopwords("english"))                       #remove stopwords
corpus <- tm_map(corpus, removeNumbers)                                           #remove numbers

#set directory for output
setwd("#your directory#")

#--------------------
#Onegram keywordlist
#--------------------
tdm <- TermDocumentMatrix(corpus)

#frequent terms for weightTf
freqtermsTf <- findFreqTerms(tdm, 50)

#frequent terms for weightTfIdf
tdm <- weightTfIdf(tdm)
freqtermsTfIdf <- findFreqTerms(tdm, 0.045)

#unique keywordvector
keywordOnegram <- c(freqtermsTf, freqtermsTfIdf)
keywordOnegram <- unique(keywordOnegram)

#--------------------
#Bigram keywordlist
#--------------------
BigramTokenizer <- function(x) { 
  unlist(lapply(ngrams(words(x), 2), paste, collapse = " "), use.names = FALSE) 
}

tdm.bigram = TermDocumentMatrix(corpus,control = list(tokenize = BigramTokenizer))
#frequent terms for weightTf
freqtermsTf <- findFreqTerms(tdm.bigram, 30)

#frequent terms for weightTfIdf
tdm.bigram <- weightTfIdf(tdm.bigram)
freqtermsTfIdf <- findFreqTerms(tdm.bigram, 0.035)

#unique keywordvector
keywordBigram <- c(freqtermsTf, freqtermsTfIdf)
keywordBigram <- unique(keywordBigram)

#---------------------------------------------
#Sentence extraction from documents in corpus 
#---------------------------------------------
sent_token_annotator <- Maxent_Sent_Token_Annotator()
corpus <- Corpus(DirSource(cname))
corpus <- tm_map(corpus, content_transformer(stripWhitespace))                    #remove whitespace

corpus_str <- paste(corpus$content, collapse = " ")
corpus_str <- as.String(corpus_str)

#perform sentence annotation
corpus_str.boundaries <- annotate(corpus_str, list(sent_token_annotator))
sentences <- corpus_str[corpus_str.boundaries]

#extract sentences containing at least one keyword from keywordOnegram and keywordBigram
j <- 1
i <- 1
sentences_onegrams <- c("Sentences Onegrams")
while (i < length(keywordOnegram))  {
  while (j < length(sentences)) {
   if (str_detect(sentences[j], keywordOnegram[i]) == TRUE){
    sentences_onegrams <- c(sentences_onegrams, sentences[j])
   }
  j <- j+1
  }
  i<-i+1
}

j <- 1
i <- 1
sentences_bigrams <- c("Sentences Bigrams")
while (i < length(keywordBigram)) {
  while (j < length(sentences)) {
   if (str_detect(sentences[j], keywordBigram[i]) == TRUE){
    sentences_bigrams <- c(sentences_bigrams, sentences[j])
   }
  j <- j+1
  }
  i<-i+1
}

#merge both sentence sets into one set
sentences_both <- c(sentences_onegrams, sentences_bigrams)

#filter sentences (regexes depend on the document!)
sentences_both <- str_replace(sentences_both, "meta \\=.*", "")
sentences_both <- str_replace(sentences_both, "list\\(.*", "")
sentences_both <- str_replace_all(sentences_both, "european journal of cancer", "")

#remove sentences showing up at least twice
sentences_both <- unique(sentences_both)

#save results to "sentences.txt"
write.csv(sentences_both, file <- paste0("#your directory for saving the sentences#", "sentences." ,"txt", sep = ""))

