#This R script takes the medical document as input and splits the main part of this document into subsections; 
#these are saved to *.txt files
#-------------------------------------------------------------------------------------------------------------

library("tm")
library("stringr")
library("SnowballC")

#--------------------------------------------------------------------------------
# Create corpus from *.txt file (use, e.g., xpdf to convert *.pdf file in shell)
#--------------------------------------------------------------------------------
cname <- file.path("#your directory containing the *.txt file#")

corpus <- Corpus(DirSource(cname))                                                #create corpus
corpus <- tm_map(corpus, content_transformer(tolower))                            #convert characters to lower case
corpus <- tm_map(corpus, removeWords, c("\\f272  european journal of cancer 46 (2010) 270<ad>283", "<ad>"))

#------------------------------------------------------------------------------------------
#Remove heading, abstract and references and write the results to doc_without_head_ref.txt 
#------------------------------------------------------------------------------------------
corpus_str <- paste(corpus$content, collapse = " ")                               #put all lines of all elements in corpus in one string "corpus_str"
corpus_str <- str_replace_all(corpus_str, "\\n", "")
corpus_str <- str_replace_all(corpus_str, "\\\\", "")
corpus_str <- str_replace_all(corpus_str, "\"", " ")

corpus_str <- str_extract(corpus_str, "introduction.*conflict of interest statement")

write.csv(corpus_str, file <- paste0("#your directory for saving the output#", paste("doc_without_head_ref", "txt", sep = ".")))

#-----------------------------------------------------------------------------------------------------------------------
#Split main document into subsections/ -> string interpolation needed!!! and save them to *.txt files
#-----------------------------------------------------------------------------------------------------------------------
#generate Julia like string interpolation; see https://www.r-bloggers.com/julia-style-string-literal-interpolation-in-r/
interpolation <- function(..., sep="", esc="#") { 

  # Change escape characters by specifying esc.

  # Break the input values into different strings cut at '#('
  x <- paste(..., sep=sep)
  x <- unlist(strsplit(x, paste0(esc,"("), fixed = TRUE))
 
  # The first element is never evaluated.
  out <- x[1]
  # Check if x has been split.
  if (length(x)>1) for (i in 2:length(x)) {
    y <- unlist(strsplit(x[i], paste0(")",esc), fixed = TRUE))
    if (x[i]==y[1])
      y <- unlist(regmatches(x[i], regexpr(")", x[i]), 
                             invert = TRUE))
    out <- paste0(out, eval(parse(text=y[1])), y[-1])
  }
  out
}

#extract subsection headings from corpus_str
subsections <- str_extract_all(corpus_str, "[0-9]\\.[0-9]{1,2}\\.[a-z\\s\\(\\)\\-]{1,80} ")

i <- 1
j <- 2

while (i <= length(subsections[[1]])){
  subsec1 <- subsections[[1]][i]
  subsec2 <- subsections[[1]][j]

  subsec <- str_extract(corpus_str, interpolation(sep = "", "(#(subsec1).*?\\s#(subsec2))"))

  if (str_detect(subsec1, interpolation(sep = "","\\/")) == TRUE){
     subsec1 <- str_replace_all(subsec1, "\\/", ".")    
    }

    write.csv(subsec, file <- paste0("#your directory for saving the output#", interpolation(sep = "", "#(subsec1).") ,"txt", sep = ""))     #write each subsection 
  i <- i+1
  j <- j+1
}

lastsection <- str_extract(corpus_str, interpolation(sep = "", "#(subsec1).{1,}"))
if (str_detect(section1, interpolation(sep = "","\\/")) == TRUE){
  section1 <- str_replace_all(section1, "\\/", ".")    
}
write.csv(lastsection, file <- paste0("#your directory for saving the output#", interpolation(sep = "", "#(subsec1).") ,"txt", sep = "")) 

consensus <- str_extract(corpus_str, "8\\. consensus\\-building process and participants \\,.*")
write.csv(consensus, file <- paste0("#your directory for saving the output#", interpolation(sep = "", "8.consensus-building process and participants.") ,"txt", sep = ""))
